@@ -1,7 +1,7 @@
 /*
  * Rufus: The Reliable USB Formatting Utility
  * Licensing Data
- * Copyright © 2011-2023 Pete Batard <pete@akeo.ie>
+ * Copyright © 2011-2025 Pete Batard <pete@akeo.ie>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -56,6 +56,10 @@ const char* additional_copyrights =
 "https://www.gnu.org/software/libcdio\\line\n"
 "GNU General Public License (GPL) v3 or later\\line\n"
 "\\line\n"
+"WIM image manipulation support from wimlib by Carl Thijssen and Eric Biggers:\\line\n"
+"https://wimlib.net\\line\n"
+"GNU General Public License (GPL) v3 or later\\line\n"
+"\\line\n"
 "Syslinux support from Syslinux by H. Peter Anvin et al.:\\line\n"
 "https://syslinux.org/\\line\n"
 "GNU General Public License (GPL) v2 or later\\line\n"
@@ -81,6 +85,10 @@ const char* additional_copyrights =
 "http://www.ridgecrop.demon.co.uk/index.htm?fat32format.htm\\line\n"
 "GNU General Public License (GPL) v2 or later\\line\n"
 "\\line\n"
+"XML parsing support from ezxml by Aaron Voisine:\\line\n"
+"https://ezxml.sourceforge.net/\\line\n"
+"MIT license\\line\n"
+"\\line\n"
 "fmifs.dll usage based on Formatx by Mark Russinovich:\\line\n"
 "https://svn.reactos.org/reactos/trunk/reactos/dll/win32/fmifs\\line\n"
 "Public Domain\\line\n"
