@@ -1,7 +1,7 @@
 /*
  * Rufus: The Reliable USB Formatting Utility
  * Virtual Disk Handling definitions and prototypes
- * Copyright © 2022-2024 Pete Batard <pete@akeo.ie>
+ * Copyright © 2022-2025 Pete Batard <pete@akeo.ie>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,44 +19,20 @@
 
 #include <stdint.h>
 #include <windows.h>
-// Temporary workaround for MinGW32 delay-loading
-// See https://github.com/pbatard/rufus/pull/2513
-#if defined(__MINGW32__)
-#undef DECLSPEC_IMPORT
-#define DECLSPEC_IMPORT __attribute__((visibility("hidden")))
-#endif
 #include <virtdisk.h>
 
 #pragma once
 
+// Someone's going to have to explain to me why trying to define ANY static inline function
+// calls in wimlib.h in DEBUG produces "C2143: syntax error: missing ';' before '{'"  with
+// MSVC, UNLESS we add the following 2 include statements here (in no particular order).
+// WHAT. THE. ACTUAL. FUCK?!?!?!
+#if defined(_MSC_VER) && defined(_DEBUG)
+#include "msapi_utf8.h"
+#include "wimlib.h"
+#endif
+
 #define WIM_MAGIC							0x0000004D4957534DULL	// "MSWIM\0\0\0"
-#define WIM_HAS_API_EXTRACT					1
-#define WIM_HAS_7Z_EXTRACT					2
-#define WIM_HAS_API_APPLY					4
-#define WIM_HAS_EXTRACT(r)					(r & (WIM_HAS_API_EXTRACT|WIM_HAS_7Z_EXTRACT))
-
-#define SECONDS_SINCE_JAN_1ST_2000			946684800
-
-#define INVALID_CALLBACK_VALUE				0xFFFFFFFF
-
-#define WIM_FLAG_RESERVED					0x00000001
-#define WIM_FLAG_VERIFY						0x00000002
-#define WIM_FLAG_INDEX						0x00000004
-#define WIM_FLAG_NO_APPLY					0x00000008
-#define WIM_FLAG_NO_DIRACL					0x00000010
-#define WIM_FLAG_NO_FILEACL					0x00000020
-#define WIM_FLAG_SHARE_WRITE				0x00000040
-#define WIM_FLAG_FILEINFO					0x00000080
-#define WIM_FLAG_NO_RP_FIX					0x00000100
-
-// Bitmask for the kind of progress we want to report in the WIM progress callback
-#define WIM_REPORT_PROGRESS					0x00000001
-#define WIM_REPORT_PROCESS					0x00000002
-#define WIM_REPORT_FILEINFO					0x00000004
-
-#define WIM_GENERIC_READ					GENERIC_READ
-#define WIM_OPEN_EXISTING					OPEN_EXISTING
-#define WIM_UNDOCUMENTED_BULLSHIT			0x20000000
 
 #define MBR_SIZE							512	// Might need to review this once we see bootable 4k systems
 
@@ -127,15 +103,10 @@ enum WIMMessage {
 	WIM_MSG_ABORT_IMAGE = -1
 };
 
-extern uint8_t WimExtractCheck(BOOL bSilent);
-extern BOOL WimExtractFile(const char* wim_image, int index, const char* src, const char* dst, BOOL bSilent);
-extern BOOL WimExtractFile_API(const char* image, int index, const char* src, const char* dst, BOOL bSilent);
-extern BOOL WimExtractFile_7z(const char* image, int index, const char* src, const char* dst, BOOL bSilent);
+extern uint32_t GetWimVersion(const char* image);
+extern BOOL WimExtractFile(const char* wim_image, int index, const char* src, const char* dst);
 extern BOOL WimApplyImage(const char* image, int index, const char* dst);
-extern char* WimMountImage(const char* image, int index);
-extern BOOL WimUnmountImage(const char* image, int index, BOOL commit);
-extern char* WimGetExistingMountPoint(const char* image, int index);
-extern BOOL WimIsValidIndex(const char* image, int index);
+extern BOOL WimSplitFile(const char* src, const char* dst);
 extern int8_t IsBootableImage(const char* path);
 extern char* VhdMountImageAndGetSize(const char* path, uint64_t* disksize);
 #define VhdMountImage(path) VhdMountImageAndGetSize(path, NULL)
